'use strict';

NodeList.prototype.forEach = Array.prototype.forEach;

function isElInView(el, fullyInView) {
    var elTop = el.getBoundingClientRect().top;
    var elBottom = el.getBoundingClientRect().bottom;
    var winHeight = window.innerHeight;

    if (fullyInView) {
        return elTop >= 0 && elBottom <= winHeight;
    }
    return elTop <= winHeight;
}

function getPageHeight() {
    var body = document.body;
    var html = document.documentElement;

    return Math.max(body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight);
}

function getScrollPos() {
    var body = document.body;
    var html = document.documentElement;

    return {
        yPos: window.pageYOffset || html.scrollTop || body.scrollTop || 0,
        xPos: window.pageXOffset || html.scrollLeft || body.scrollLeft || 0
    };
}

function stickyCTA() {
    var defaults = {};
    var dom = {};

    function cacheData(entry) {
        dom.footer = document.querySelector('.footer-upper');
        if (dom.footer) dom.footerPos = getPageHeight() - dom.footer.getBoundingClientRect().height;
        dom.bottomCTA = document.querySelector('.main-footer');
        if (dom.bottomCTA) dom.bottomCTAPos = getPageHeight() - (dom.footer.getBoundingClientRect().height + dom.bottomCTA.getBoundingClientRect().height) - 67 + 'px';
        dom.stickyCTA = document.querySelector('.js-sticky-cta');
        dom.stickyCTABCR = dom.stickyCTA.getBoundingClientRect();
        defaults.enter = getPageHeight() * entry.enter;
        defaults.exit = getPageHeight() * entry.exit;
    }

    function modifyFooter() {
        var lowerPos = dom.bottomCTAPos || dom.footerPos;
        if (lowerPos < defaults.exit) {
            if (dom.bottomCTA) {
                dom.bottomCTA.style.marginTop = dom.stickyCTABCR.height - 1 + 'px';
            } else {
                dom.footer.style.marginTop = dom.stickyCTABCR.height - 1 + 'px';
            }
            defaults.exit += dom.stickyCTABCR.height;
        }
    }

    function anchorEl(el, top) {
        var elHeight = Math.max(el.clientHeight, el.offsetHeight, el.scrollHeight);
        el.classList.remove('is-sticky');
        el.classList.add('is-stuck');
        el.style.setProperty('top', top - elHeight + 1 + 'px');
    }

    function releaseEl(el) {
        el.classList.remove('is-stuck');
        el.classList.add('is-sticky');
        el.style.setProperty('top', '');
    }

    function showCta() {
        if (!dom.stickyCTA.classList.contains('is-visible')) {
            dom.stickyCTA.classList.add('is-visible');
        }
    }

    function hideCta() {
        if (dom.stickyCTA.classList.contains('is-visible')) {
            dom.stickyCTA.classList.remove('is-visible');
        }
    }

    function checkVis() {
        var html = document.body;
        if (defaults.enter <= getScrollPos().yPos + window.innerHeight) {
            showCta();
        }
        if (defaults.exit < getScrollPos().yPos || getScrollPos().yPos + window.innerHeight < defaults.enter) {
            hideCta();
        }
        var lowerEl = dom.bottomCTA || dom.footer;

        if (!lowerEl) return;

        if (isElInView(lowerEl)) {
            anchorEl(dom.stickyCTA, lowerEl.offsetTop);
        } else {
            releaseEl(dom.stickyCTA);
        }
    }

    function setEvents() {
        window.addEventListener('scroll', checkVis, false);
    }

    function init() {
        var entry = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { enter: 0, exit: 1 };

        cacheData(entry);
        if (dom.bottomCTA || dom.footer) modifyFooter();
        checkVis();
        setEvents();
    }

    return {
        init: init
    };
}

window.sticky = stickyCTA();